//-----------------------------------------------------------------------------
// File: AntiAlias.fx
//
// Desc: The DX9 Effect File for Blurring with the GroupAntiAlias node 
//
// 
// Copyright (c) 2007-2007, Bitmanagement Software GmbH. All rights reserved.
// based on Blurr  shader from Cecile Muller, webmaster .at. wildpeaks.com
//   
//-----------------------------------------------------------------------------
// Nvidia Geforce FX:  Device max profiles: vs_2_a, ps_2_a 


        
// Transformation matrices    
float4x4 World 		: WORLD;         
float4x4 View 		: VIEW;     
float4x4 Projection 	: PROJECTION; 

// screen parameters
float2 WindowSize : VIEWPORTPIXELSIZE;
float2 WindowOffset = { 0.0, 0.0 };
float2 ScreenSize = { 1600.0f, 1200.0f };
float2 SceneMapPixelDelta = { 1.0/1024, 1.0/768 };


// left scene buffer 
texture SceneMap0 : RENDERCOLORTARGET
< 
    float2 ViewportRatio = { 1.0, 1.0 };
    int MIPLEVELS = 1;
    string format = "X8R8G8B8";
>;

//--------------------------------------------------------------------------------------
// Texture samplers
//--------------------------------------------------------------------------------------


sampler SceneMapSampler = sampler_state 
{
    texture = <SceneMap0>;
    AddressU  = CLAMP;   
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = NONE;
    MINFILTER = POINT; // LINEAR;
    MAGFILTER = POINT; // LINEAR;
    //MINFILTER = LINEAR; // LINEAR;
    //MAGFILTER = LINEAR; // LINEAR;
};



//--------------------------------------------------------------------------------------
// Name: VS
// Type: Vertex Shader
// Desc: Projection transform
//--------------------------------------------------------------------------------------
void VS( float4 posObj: POSITION,
         float4 colorIn: COLOR0,
         float2 texCoordIn: TEXCOORD0,
         
         out float4 posProj: POSITION,
         out float4 colorOut: COLOR0,
         out float2 texCoordOut: TEXCOORD0 )
{
    // Transform the position into projected space for display and world space for lighting
    posProj = mul( posObj, World ); // is identity anyway
    posProj = mul( posProj, View );
    posProj = mul( posProj, Projection );
  
    // pass vertex current color
    colorOut = colorIn;
     
    // Pass the texture coordinate
    texCoordOut = texCoordIn; // +SceneMapPixelDelta*0.5;
    texCoordOut.y -= SceneMapPixelDelta.y;
}



//--------------------------------------------------------------------------------------
// Name: PS_Blurr
// Type: Pixel Shader
// Desc: Modulate the texture by modulateColor
//--------------------------------------------------------------------------------------



float4 wcolor = {1.0/4.0,1.0/4.0,1.0/4.0,1.0/4.0 };


// using n pathes, with texture and modulate factor as arguments 
void PS_Blurr( 
	 float4 colorIn: COLOR0,
         float2 texCoord: TEXCOORD0,
         out float4 colorOut: COLOR0,
	      uniform sampler Sampler,
	      uniform float4 modulateColor,
        uniform float2 offset          
         
	 )
{  
    colorOut = colorIn;

    // Sample and modulate the texture
     colorOut *= tex2D( Sampler, texCoord  + offset);
     colorOut *= modulateColor;
}

// using one path with ps_2_0 model 
void PS_Blurr_SinglePath( 
	 float4 colorIn: COLOR0,
         float2 texCoord: TEXCOORD0,
         out float4 colorOut: COLOR0
	 )
{  
    //colorOut = colorIn;

    // Sample and modulate the texture
/*    
    float w = 1.0/5.0;
    // filter 5 pixels 
    colorOut = w*tex2D( SceneMapSampler, texCoord);
    colorOut += w*tex2D( SceneMapSampler,texCoord+float2(SceneMapPixelDelta.x,0));
    colorOut += w*tex2D( SceneMapSampler,texCoord+float2(-SceneMapPixelDelta.x,0));
    colorOut += w*tex2D( SceneMapSampler,texCoord+float2(0.0,SceneMapPixelDelta.y));
    colorOut += w*tex2D( SceneMapSampler,texCoord+float2(0.0,-SceneMapPixelDelta.y));
*/    

    float w = 1.0/4.0;
    // filter 4pixels 
    colorOut = w*tex2D( SceneMapSampler, texCoord);
    colorOut += w*tex2D( SceneMapSampler,texCoord+float2(SceneMapPixelDelta.x,0));
    colorOut += w*tex2D( SceneMapSampler,texCoord+float2(SceneMapPixelDelta.x,-SceneMapPixelDelta.y));
    colorOut += w*tex2D( SceneMapSampler,texCoord+float2(0.0,-SceneMapPixelDelta.y));


/*
    float4 color;
  
    float w = 1.0/5.0;
    // filter 5 pixels 
    //w=0.5;
    color = w*tex2D( SceneMapSampler, texCoord);
    //w= 0.5/4;
    color += w*tex2D( SceneMapSampler,texCoord+float2(SceneMapPixelDelta.x,0));
    color += w*tex2D( SceneMapSampler,texCoord+float2(0.0,SceneMapPixelDelta.y));
    color += w*tex2D( SceneMapSampler,texCoord+float2(-SceneMapPixelDelta.x,0));
    color += w*tex2D( SceneMapSampler,texCoord+float2(0.0,-SceneMapPixelDelta.y));
    colorOut=color;
*/    
    
    //colorOut.r = 1.0;

}


// box filter with radius strength

float4 BoxBlur(
  uniform int  strength,
  float2 Tex : TEXCOORD0): COLOR0 
  {
	float4 Color;
	float4 OriginalColor;
	// OriginalColor = tex2D( SceneMapSampler, float2(Tex.x, Tex.y) );
	Color = float4(0,0,0,0); // OriginalColor;

	// Declaring inside instead
	//int strength = 3; // don't set too high 
	float offset = 0.001;
	float blend = 0.5;
	float sharpening = 5.0f;


	// Makes the box blurring
	if ((strength > 0) && (strength < 64)) 
  {
		for (int i = 0; i < strength; i++) {
			for (int j = 0; j < strength; j++) {
				Color += tex2D( SceneMapSampler, float2(Tex.x + (i-strength*0.5) * SceneMapPixelDelta.x, Tex.y - (j-strength*0.5) * SceneMapPixelDelta.y) );
			}
		}
		Color = Color / (strength * strength);
	}
	else 
    Color = tex2D( SceneMapSampler, float2(Tex.x, Tex.y) );


	// The sharpening stage - from http://www.facewound.com/tutorials/shader1/
	//Color -= tex2D( SceneMapSampler, float2(Tex.x, Tex.y) + SceneMapPixelDelta) * sharpening;
	//Color += tex2D( SceneMapSampler, float2(Tex.x, Tex.y) - SceneMapPixelDelta) * sharpening;


	return Color;
}



//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Name: PS_Blurr
// Type: Technique
// Desc: Renders  scene to render target
//-----------------------------------------------------------------------------



// 1 Pass Box filter radius 3
technique Blurr_Box
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();    
        //PixelShader = compile ps_2_a BoxBlur(4);
        PixelShader = compile ps_2_0 BoxBlur(3);   
           
    }
}

// 1 Pass 
technique Blurr_SinglePath
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();    
        PixelShader = compile ps_2_0 PS_Blurr_SinglePath();   
    }
}

// 4 Pass with blending -  is stripy 
technique Blurr_MultiPath
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();    
        PixelShader = compile ps_2_0 PS_Blurr( SceneMapSampler, wcolor,float2(0,0));   
 	      AlphaBlendEnable = false;         
        
    }
    
    pass P1
    {
        VertexShader = compile vs_1_1 VS();    
        PixelShader = compile ps_2_0 PS_Blurr( SceneMapSampler, wcolor,float2(SceneMapPixelDelta.x,0) );   
        // enable alpha blending
        AlphaBlendEnable = TRUE;
        SrcBlend         = ONE;
        DestBlend        = ONE; 	
    }
    pass P2
    {
        VertexShader = compile vs_1_1 VS();    
        PixelShader = compile ps_2_0 PS_Blurr( SceneMapSampler, wcolor,float2(SceneMapPixelDelta.x,-SceneMapPixelDelta.y) );   
    }
    pass P3
    {
        VertexShader = compile vs_1_1 VS();    
        PixelShader = compile ps_2_0 PS_Blurr( SceneMapSampler, wcolor,float2(0,-SceneMapPixelDelta.y) );   
    }
    
}


// identity 1:1 
technique Blurr_1
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();    
        PixelShader = compile ps_2_0 PS_Blurr( SceneMapSampler, float4(1,1,1,1),float2(SceneMapPixelDelta.x*0.5,SceneMapPixelDelta.y*0.5) );    
        PixelShader = compile ps_2_0 PS_Blurr( SceneMapSampler, float4(1,1,1,1),float2(0,0) );    
    }
}

